require 'spec_helper'

module Jisota
  describe 'calling upload blocks', type: :acceptance do
    example "update and create blocks inside package with params" do
      config = Jisota.config do
        package :upload_file do
          param :file
          run do
            upload from: file, to: "foo" do
              create { cmd %Q{echo "File #{file} created"} }
              update { cmd %Q{echo "File #{file} updated"} }
            end
          end
        end

        role :app do
          upload_file "spec/test_files/foo"
        end

        server host, user: user, roles: :app, key: key
      end

      create_ssh_session { |ssh_session| ssh_session.command("rm foo") }
      Jisota.run(config, logger: logger)

      create_ssh_session { |ssh_session| ssh_session.command("echo omg > foo") }
      Jisota.run(config, logger: logger)

      expect(logger.stdout.writes).to include("File spec/test_files/foo created\n")
      expect(logger.stdout.writes).to include("File spec/test_files/foo updated\n")
    end
  end
end
