require 'spec_helper'

module Jisota
  describe Collection do
    describe "#add" do
      it "adds the item" do
        collection = Collection.new
        item = double(key: :foo)
        collection.add(item)

        expect(collection[:foo]).to eq(item)
        expect(collection.size).to eq(1)
      end

      it "will not allow duplicate key" do
        collection = Collection.new
        item = double(key: :foo)
        collection.add(item)

        expect { collection.add(item) }.to raise_error(Collection::DuplicateKeyError)
      end
    end

    describe "#merge" do
      it "returns a new collection with all items" do
        collection_1 = Collection.new
        collection_1.add(double(key: :foo, value: 1))
        collection_1.add(double(key: :bar, value: 2))

        collection_2 = Collection.new
        collection_2.add(double(key: :bar, value: 3))
        collection_2.add(double(key: :baz, value: 4))

        result = collection_1.merge(collection_2) { |_, left, _| left }

        expect(collection_1.size).to eq(2)
        expect(collection_2.size).to eq(2)
        expect(result.size).to eq(3)

        expect(result.map(&:value)).to eq([1, 2, 4])
      end
    end

    describe "#first" do
      it "returns the first item in the collection" do
        collection = Collection.new
        collection.add(double(key: :foo))
        collection.add(double(key: :bar))

        expect(collection.first.key).to eq(:foo)
      end
    end
  end
end
