require 'spec_helper'

module Jisota
  describe CompositeScript do
    describe "#scripts" do
      it "initializes to empty array" do
        expect(CompositeScript.new.scripts).to eq([])
      end
    end

    describe "execute" do
      let(:ssh_session) { instance_double(SSHSession) }
      let(:context) { ScriptContext.new(ssh_session: ssh_session) }
      let(:script) { CompositeScript.new }

      it "executes all scripts" do
        inner_1 = double(execute: true)
        inner_2 = double(execute: true)
        script.scripts << inner_1 << inner_2
        result = script.execute(context)

        expect(inner_1).to have_received(:execute).with(context)
        expect(inner_2).to have_received(:execute).with(context)
        expect(result).to be true
      end

      it "stops execution if one script fails" do
        inner_1 = double(execute: false)
        inner_2 = double(execute: true)
        script.scripts << inner_1 << inner_2
        result = script.execute(context)

        expect(inner_1).to have_received(:execute).with(context)
        expect(inner_2).to_not have_received(:execute).with(context)
        expect(result).to be false
      end
    end
  end
end
