require 'spec_helper'

module Jisota
  describe Configuration do
    describe "#packages" do
      it "can manage packages" do
        package = instance_double(Package, name: :foo, key: :foo)
        config = Configuration.new
        config.add_package(package)
        expect(config.get_package(package.name)).to eq(package)
      end
    end

    describe "#roles" do
      it "can manage roles" do
        role = instance_double(Role, name: :foo, key: :foo)
        config = Configuration.new
        config.add_role(role)
        expect(config.get_role(role.name)).to eq(role)
      end
    end

    describe "#servers" do
      it "can manage server" do
        server = instance_double(Server)
        config = Configuration.new
        config.add_server(server)

        count = 0
        config.each_server do |a_server|
          count += 1
          expect(a_server).to eq(server)
        end
        expect(count).to eq(1)
      end
    end

    describe "initialize with DSL" do
      describe "#package" do
        it "reads the package" do
          config = Configuration.new do
            package :foo do
              description "bar"
            end
          end
          package = config.packages.first

          expect(config.packages.size).to eq(1)
          expect(package.name).to eq(:foo)
          expect(package.description).to eq("bar")
        end
      end

      describe "#role" do
        it "creates a role" do
          config = Configuration.new do
            role(:app) { cmd "touch foo" }
          end
          role = config.roles.first
          expect(role.name).to eq(:app)
          expect(role.script_block).to be_a(ScriptBlock)
        end
      end

      describe "#server" do
        it "creates a server" do
          config = Configuration.new do
            server "example.org", user: "foo", roles: [:app, :db]
          end
          server = config.servers.first

          expect(config.servers.size).to eq(1)
          expect(server.host).to eq("example.org")
          expect(server.user).to eq("foo")
          expect(server.roles).to eq([:app, :db])
        end
      end
    end
  end
end
