require 'spec_helper'

module Jisota
  describe Output do
    let(:logger) { Output.new(stdout: stdout, stderr: stderr, verbose: verbose) }
    let(:stdout) { instance_double(IO, write: nil, flush: nil) }
    let(:stderr) { instance_double(IO, write: nil, flush: nil) }
    let(:verbose) { true }

    describe "initialization" do
      it "initializes with options" do
        logger = Output.new(stdout: :out, stderr: :err, verbose: false)
        expect(logger.stdout).to eq(:out)
        expect(logger.stderr).to eq(:err)
        expect(logger.verbose).to be false
      end
    end

    describe '#info' do
      it "writes to stdout" do
        logger.info("foo")
        expect(stdout).to have_received(:write).with("foo")
      end
    end

    describe "#error" do
      it "writes to stderr" do
        logger.error("foo")
        expect(stderr).to have_received(:write).with("foo")
      end
    end

    describe "#warn" do
      context "verbose" do
        let(:verbose) { true }
        it "outputs" do
          logger.warn("foo")
          expect(stderr).to have_received(:write).with("foo")
        end
      end

      context "not verbose" do
        let(:verbose) { false }
        it "outputs" do
          logger.warn("foo")
          expect(stderr).to_not have_received(:write).with("foo")
        end
      end
    end

    describe "#system_message" do
      it "outputs message" do
        logger.system_message("foo")
        expect(stdout).to have_received(:write).with("-----> foo\n")
      end
    end

    describe "#indent" do
      it "indents subsequent output" do
        logger.indent
        logger.system_message("foo")
        expect(stdout).to have_received(:write).with("----->   foo\n")

        logger.outdent
        logger.system_message("foo")
        expect(stdout).to have_received(:write).with("-----> foo\n")
      end

      context "with a block" do
        it "indents logs inside block" do
          result = logger.indent do
            logger.system_message("foo")
            42
          end
          logger.system_message("bar")
          expect(stdout).to have_received(:write).with("----->   foo\n")
          expect(stdout).to have_received(:write).with("-----> bar\n")
          expect(result).to eq(42)
        end
      end
    end

  end
end
