require 'spec_helper'

module Jisota
  describe PackageScript do
    describe "#execute" do
      let(:ssh_session) { instance_double(SSHSession, command: 0) }
      let(:context) { ScriptContext.new(ssh_session: ssh_session) }

      it "executes each script from the block" do
        package = Package.new do
          run { cmd "foo" }
        end
        package_script = PackageScript.new(package)
        result = package_script.execute(context)

        expect(result).to be true
        expect(ssh_session).to have_received(:command).with("foo", anything)
      end

      it "parses args" do
        package = Package.new do
          param :bar
          run { cmd "foo #{bar}" }
        end
        package_script = PackageScript.new(package, ["baz"])
        package_script.execute(context)

        expect(ssh_session).to have_received(:command).with("foo baz", anything)
      end

      it "uses verify block" do
        package = Package.new do
          run { cmd "foo" }
          verify { cmd "bar" }
        end
        package_script = PackageScript.new(package)
        package_script.execute(context)

        expect(ssh_session).to have_received(:command).with("bar", anything)
        expect(ssh_session).to_not have_received(:command).with("foo", anything)
      end
    end

    describe "#to_s" do
      it "outputs a nice representation of the script" do
        package = Package.new(:nginx)

        expect(PackageScript.new(package, []).to_s).to eq("nginx")
        expect(PackageScript.new(package, ["foo"]).to_s).to eq('nginx "foo"')
        expect(PackageScript.new(package, ["foo", 42]).to_s).to eq('nginx "foo", 42')
        expect(PackageScript.new(package, [{ foo: 42 }]).to_s).to eq("nginx {:foo=>42}")
      end
    end
  end
end
