require 'spec_helper'

module Jisota
  describe Package do
    describe '#name' do
      it "has a name" do
        package = Package.new
        package.name = :foo
        expect(package.name).to eq(:foo)
      end
    end

    describe "#params" do
      it "has params" do
        package = Package.new
        expect(package.params).to eq([])
      end

      it "can add param" do
        package = Package.new
        package.params << :foo
        expect(package.params).to eq([:foo])
      end
    end

    describe "#description" do
      it "has one" do
        package = Package.new
        package.description = :foo
        expect(package.description).to eq(:foo)
      end
    end

    describe Package::DSL do
      it "initializes using DSL" do
        package = Package.new(:my_awesome_package) do
          description "Does cool stuff"
          param :foo
          param :bar, default: 42

          run { cmd "touch #{foo}" }
          verify { cmd "[ -f foo ]" }
        end

        expect(package.name).to eq(:my_awesome_package)
        expect(package.description).to eq("Does cool stuff")
        expect(package.params.size).to eq(2)
        expect(package.params.last.name).to eq(:bar)
        expect(package.params.last.options).to eq({ default: 42 })
        expect(package.sections.size).to eq(1)
        expect(package.sections.first.run_block).to be_a(ScriptBlock)
        expect(package.sections.first.verify_block).to be_a(ScriptBlock)
      end

      it "has many run and verify blocks" do
        package = Package.new(:something) do
          run { cmd "touch foo" }
          verify { cmd "[ -f foo ]" }

          run { cmd "touch bar" }

          run { cmd "touch baz" }
          verify { cmd "[ -f baz ]" }
        end

        expect(package.sections.size).to eq(3)
      end

      it "must have a run block per verify block" do
        expect {
          package = Package.new { verify { } }
        }.to raise_exception(Package::InvalidBlockOrder)

        expect {
          package = Package.new do
            run { }
            verify { }
            verify { }
          end
        }.to raise_exception(Package::InvalidBlockOrder)
      end

      it "accepts section names" do
        package = Package.new(:foo) do
          section "do amazing stuff"
          run {}
        end

        expect(package.sections.size).to eq(1)
        expect(package.sections.first.description).to eq("do amazing stuff")
      end
    end
  end
end
