require 'spec_helper'

describe Jisota::ParamParser do
  subject { parser.parse() }
  let(:parser) { Jisota::ParamParser.new(params, args) }
  let(:params) { [] }
  let(:args) { [] }

  def should_raise(error)
    expect { subject }.to raise_exception(error)
  end

  context "no params" do
    context "no args" do
      it { should eq Hash.new }
    end

    context "implicit arg" do
      let(:args) { [42] }
      it { should_raise(Jisota::ParameterError) }
    end

    context "hash arg" do
      let(:args) { [ { foo: 42 } ] }
      it { should_raise(Jisota::ParameterError) }
    end
  end

  context "param with no options" do
    let(:params) { [Jisota::Param.new(:foo)] }

    context "no args" do
      it { should eq({foo: nil}) }
    end

    context "implicit arg" do
      let(:args) { [42] }
      it { should eq({foo: 42}) }
    end

    context "repeated implicit arg" do
      let(:args) { [42, { foo: 42 }] }
      it { should_raise(Jisota::ParameterError) }
    end

    context "hash arg" do
      let(:args) { [ { foo: 42 } ] }
      it { should eq({foo: 42}) }
    end
  end

  context "param with required: true" do
    let(:params) { [Jisota::Param.new(:foo, required: true)] }

    context "no args" do
      it { should_raise(Jisota::ParameterError) }
    end

    context "implicit arg" do
      let(:args) { [42] }
      it { should eq({foo: 42}) }
    end

    context "hash arg" do
      let(:args) { [ { foo: 42 } ] }
      it { should eq({foo: 42}) }
    end
  end

  context "param with default" do
    let(:params) { [Jisota::Param.new(:foo, default: 48)] }

    context "no args" do
      it { should eq({foo: 48}) }
    end

    context "implicit arg" do
      let(:args) { [42] }
      it { should eq({foo: 42}) }
    end

    context "hash arg" do
      let(:args) { [ { foo: 42 } ] }
      it { should eq({foo: 42}) }
    end
  end

  context "param with splat: true" do
    let(:params) { [Jisota::Param.new(:foo, splat: true)] }

    context "no args" do
      it { should eq({foo: []}) }
    end

    context "implicit arg" do
      let(:args) { [42] }
      it { should eq({foo: [42]}) }
    end

    context "hash arg" do
      let(:args) { [ { foo: 42 } ] }
      it { should_raise(Jisota::ParameterError) }
    end
  end
end
