require 'spec_helper'

module Jisota
  describe Provisioner do
    describe "#run" do

      let(:ssh_engine) do
        instance_double(SSHEngine).tap do |ssh|
          allow(ssh).to receive(:start).with({user: "john", host: "123.456.789.0"}).and_yield(ssh_session)
        end
      end
      let(:ssh_session) { instance_double(SSHSession, command: 0) }

      it "executes a script" do
        config = Configuration.new do
          role :app do
            cmd "touch foo"
          end
          server "123.456.789.0", user: "john", roles: :app
        end
        config.ssh_engine = ssh_engine

        provisioner = Provisioner.new
        provisioner.run(config, NilOutput.new)

        expect(ssh_session).to have_received(:command).with("touch foo", anything)
      end
    end
  end
end
