require 'spec_helper'

module Jisota
  describe ScriptBlock do
    describe "#block" do
      it "is initialized with a block" do
        script_block = ScriptBlock.new { foo }
        expect(script_block.block).to be_a(Proc)
      end
    end

    describe "#evaluate" do
      it "creates a script from the block" do
        script_block = ScriptBlock.new do
          cmd "foo"
          upload from: "x", to: "y"
        end

        script = script_block.evaluate

        expect(script).to be_a(CompositeScript)
        expect(script.scripts.size).to eq(2)
        expect(script.scripts[0]).to be_a(CommandScript)
        expect(script.scripts[0].command).to eq("foo")
        expect(script.scripts[1]).to be_a(FileScript)
        expect(script.scripts[1].from).to eq("x")
        expect(script.scripts[1].to).to eq("y")
      end

      it "parses args" do
        script_block = ScriptBlock.new { cmd "touch #{foo}" }
        params = { foo: "bar" }
        context = ScriptContext.new(params: params)
        script = script_block.evaluate(context)

        expect(script.scripts.first.command).to eq("touch bar")
      end

      it "calls packages" do
        script_block = ScriptBlock.new { touch "file" }
        packages = Collection.new
        packages << Package.new(:touch)
        context = ScriptContext.new(packages: packages)

        script = script_block.evaluate(context)
        script = script.scripts.first

        expect(script).to be_a(PackageScript)
        expect(script.package).to be(packages.first)
        expect(script.args).to eq(["file"])
      end
    end
  end
end
