require 'spec_helper'

module Jisota
  describe SSHSession do

    let(:wrapper) { SSHSession.new(session, scp_engine: scp_engine) }
    let(:session) { instance_double(Net::SSH::Connection::Session, loop: nil) }
    let(:channel) { instance_double(Net::SSH::Connection::Channel, on_data: nil, on_extended_data: nil, on_request: nil) }
    let(:logger) { instance_double(Output, system_message: nil, error: nil) }
    let(:scp_engine) { class_double(Net::SCP, new: scp_instance) }
    let(:scp_instance) { instance_double(Net::SCP, upload!: nil) }
    let(:exit_code) { 0 }
    before do
      allow(session).to receive(:open_channel).and_yield(channel)
      allow(channel).to receive(:exec).and_yield
      allow(channel).to receive(:on_request).with("exit-status").and_yield(nil, instance_double(Net::SSH::Buffer, read_long: exit_code))
    end

    describe "#command" do
      it "executes command through a channel" do
        result = wrapper.command("touch foo", logger)

        expect(channel).to have_received(:exec).with("touch foo")
        expect(result).to eq(0)
        expect(logger).to have_received(:system_message).with("Executing touch foo")
      end

      describe "error in executing command" do
        let(:exit_code) { 1 }
        it "logs error" do
          result = wrapper.command("touch foo", logger)

          expect(channel).to have_received(:exec).with("touch foo")
          expect(result).to eq(1)
          expect(logger).to have_received(:system_message).with("Executing touch foo")
          expect(logger).to have_received(:error).with("Error running touch foo:")
        end
      end
    end

    describe "#upload" do
      it "delegates to scp engine" do
        result = wrapper.upload(from: "spec/test_files/foo", to: "bar")

        expect(scp_instance).to have_received(:upload!) do |from, to, options|
          expect(from).to eq("spec/test_files/foo")
          expect(to).to eq("bar")
        end
      end
    end

  end
end
